class VirtualNode

  attr_accessor :original_nodes, :functions, :draw_method
  def initialize(nodes)
    nodes = [nodes] unless Array === nodes
    nodes.each{|node|
      if Node === node
        unless node.variable?
          raise "node must be variable"
        end
      elsif ! VirtialNode === node
        raise "node is invalid"
      end
    }
    @original_nodes = nodes
    @num_vars = nodes.length
    @functions = Array.new
    @draw_method = nil
  end

  def function(func,args=nil)
    unless Function === func
      raise "function is invalid"
    end
    if @draw_method
      raise "cannot apply function after drawing"
    end
    unless @num_vars%func.nvars == 0
      raise "wrong number of variables (#{@num_vars}%#{func.nvars} != 0)"
    end
    if args && args.length > (al = func.function_arguments.length)
      raise "wrong number of arguments (#{args.length} for #{al})"
    end
    @functions.push [func,args]
    @num_vars = func.function_outputs.length * (@num_vars/func.nvars)
    return self
  end

  def [](*ind)
    if ind.length == 0
      raise "index is invalid"
    end
    if ind.max >= @num_vars
      raise "index out of range"
    end
    ind.each{|i|
      unless Integer === i
        raise "indices must be integer"
      end
    }
    @functions.push ind
    @num_vars = ind.length
    return self
  end

  def type
    @draw_method ? "draw" : "analysis"
  end

  def gen_code(nvars,vars,id=0)
    if id == 0
      vars_org = vars
      vars = Array.new
    end
    script = "gphyses#{id} = Array.new\n"
    @original_nodes.each_with_index{|node,i|
      case node
      when Node
        script += "gphyses#{id}.push gphyses[#{vars.length}]"
        vars.push node.entity
      when VirtualNode
        id2 = id*10+i
        script += node.gen_code(var,id2)
        script += "gphyses#{id} += proc#{id2}.call\n"
      else
        raise "[BUG] node is invalid"
      end
      if id == 0
        if vars.length == nvars
          vars_org.push vars
          vars = Array.new
        elsif vars.length > nvars
          raise "[BUG} vars.length is invalid"
        end
      end
    }
    script += "proc#{id} = Proc.new{\n"
    @functions.each{|func|
      if Function === func[0]
        func, args = func
        nvars = func.nvars
        fargs = func.function_arguments
        ary = Array.new
        nvars.times{|i| ary << "gphys#{i}"}
        args ||= Array.new
        fargs.each_with_index{|fa,i|
          ary << "arg#{i}"
          args[i] = (args[i] || fa.default).inspect
        }
        script +=<<"EOF"
          new_gphyses = Array.new
          (gphyses#{id}.length/#{nvars}).times{|i|
             proc = Porc.new{|#{ary.join(",")}|
                #{func.script}
             }
             new_gphyses += proc.call(#{s=Array.new;nvars.times{|i|s.push("gphyses"+id.to_s+"["+(i*nvars).to_s+"+i]")};args.each{|arg|s.push arg};s.join(",")})
          }
          gphyses#{id} = new_gphyses
EOF
      else
        script += "gphyses#{id} = gphyes#{id}[#{func.join(",")}]\n"
      end
    }
    script +=<<"EOF"
       return gphyses#{id}
     }
EOF
    if id==0
      script += "gphyses = proc#{id}.call\n"
    end
    return script
  end

  def get
    if dm = @draw_method
      vars = Array.new
      script = gen_code(dm.nvars,vars)
      plot = Hash.new
      plot[:method] = dm.vizshot_method.to_sym
      (@num_vars/dm.nvars).times{|i|
        viz = NumRu::VizShot.new(:iwidth => size[0], :iheight => size[1], :basename => basename)
        viz.set_fig("itr" => draw_projection, "viewport" => viewport)
        viz.set_tone("tonf" => true) unless draw_projection == 5
        plot[:variables] = vars[i].collect{|var| var.path}
        viz.plot(plot.dup)
        #      viz.dump_code( nil, false, :image_dump => true)
        code = viz.gen_code( nil, false, :image_dump => true)  # test
      }
    else
      code = <<"EOF"
require "numru/gphys"
begin
 require File.dirname(__FILE__)+"/../../../lib/gphys_gfdnavi"
rescue LoadError
end
begin
 require File.dirname(__FILE__)+"/../../../../lib/gphys_gfdnavi"
rescue LoadError
end
include NumRu

$SAFE = 3

GPhys::read_size_limit_2 = #{GPHYS_READ_SIZE_LIMIT_2.inspect}
GPhys::read_size_limit_1 = #{GPHYS_READ_SIZE_LIMIT_1.inspect}

EOF
      
      code += <<"EOF"
#{self.gen_code}

ofname = ARGV.shift || "output.nc"
file = NetCDF.create(ofname)
begin
  $SAFE=1
  gphyses.each{|gphys|
    GPhys::IO.write(file,gphys)
  }
ensure
  file.close
end

EOF
    end
  end

  def draw(dm,opts=nil)
    unless DrawMethod === dm
      raise "draw method is invalid"
    end
    unless @num_vars%dm.nvars == 0
      raise "wrong number of variables (#{@num_vars}%#{dm.nvars}!=0)"
    end
    @draw_method = [dm,opts]
    return self
  end

  def to_xml(arg={})
    uri_prefix = arg.delete(:uri_prefix)
    nodes = Array.new
    @original_nodes.each{|on|
      case on
      when Node
        path = on.path
        nodes.push( {"path" => path, "uri" => File.join(uri_prefix, "data", path+".xml")} )
      when VirtualNode
        nodes.push node
      end
    }
    funcs = Array.new
    @functions.each{|func|
      case func[0]
      when Function
        path = func[0].path
        hash = {"path" => path, "uri" => File.join(uri_prefix, "data", path+".xml")}
        hash["arguments"] = func[1] if func[1]
        funcs.push hash
      when Integer
        funcs.push( {"indices" => func} )
      end
    }
    hash = {"nodes" => nodes, "functions" => funcs}
    if @draw_method
      path = @draw_method[0].path
      hash["draw_method"] = {"path" => @draw_method[0].path, "uri" => File.join(uri_prefix, "data", path+".xml")}
      hash["draw_method"]["options"] = @draw_method[1] if @draw_method[1]
    end
    return hash.to_xml(*arg)
  end
end

